@@ -65,6 +65,30 @@ R_LIB_VERSION_HEADER (r_bin);
 #define R_BIN_REQ_HEADER   0x2000000
 #define R_BIN_REQ_LISTPLUGINS   0x4000000
 
+/* RBinSymbol->method_flags : */
+#define R_BIN_METH_CLASS 0x0000000000000001L
+#define R_BIN_METH_STATIC 0x0000000000000002L
+#define R_BIN_METH_PUBLIC 0x0000000000000004L
+#define R_BIN_METH_PRIVATE 0x0000000000000008L
+#define R_BIN_METH_PROTECTED 0x0000000000000010L
+#define R_BIN_METH_INTERNAL 0x0000000000000020L
+#define R_BIN_METH_OPEN 0x0000000000000040L
+#define R_BIN_METH_FILEPRIVATE 0x0000000000000080L
+#define R_BIN_METH_FINAL 0x0000000000000100L
+#define R_BIN_METH_VIRTUAL 0x0000000000000200L
+#define R_BIN_METH_CONST 0x0000000000000400L
+#define R_BIN_METH_MUTATING 0x0000000000000800L
+#define R_BIN_METH_ABSTRACT 0x0000000000001000L
+#define R_BIN_METH_SYNCHRONIZED 0x0000000000002000L
+#define R_BIN_METH_NATIVE 0x0000000000004000L
+#define R_BIN_METH_BRIDGE 0x0000000000008000L
+#define R_BIN_METH_VARARGS 0x0000000000010000L
+#define R_BIN_METH_SYNTHETIC 0x0000000000020000L
+#define R_BIN_METH_STRICT 0x0000000000040000L
+#define R_BIN_METH_MIRANDA 0x0000000000080000L
+#define R_BIN_METH_CONSTRUCTOR 0x0000000000100000L
+#define R_BIN_METH_DECLARED_SYNCHRONIZED 0x0000000000200000L
+
 enum {
 	R_BIN_SYM_ENTRY,
 	R_BIN_SYM_INIT,
@@ -91,7 +115,8 @@ enum {
 	R_STRING_TYPE_DETECT = '?',
 	R_STRING_TYPE_ASCII = 'a',
 	R_STRING_TYPE_UTF8 = 'u',
-	R_STRING_TYPE_WIDE = 'w',
+	R_STRING_TYPE_WIDE = 'w', // utf16 / widechar string
+	R_STRING_TYPE_WIDE32 = 'W', // utf32
 	R_STRING_TYPE_BASE64 = 'b',
 };
 
@@ -285,7 +310,6 @@ typedef struct r_bin_xtr_plugin_t {
 	char *license;
 	int (*init)(void *user);
 	int (*fini)(void *user);
-	bool (*check)(RBin *bin);
 // XXX: ut64 for size is maybe too much, what about st64? signed sizes are useful for detecting errors
 	bool (*check_bytes)(const ut8 *bytes, ut64 sz);
 	RBinXtrData * (*extract_from_bytes)(RBin *bin, const ut8 *buf, ut64 size, int idx);
@@ -306,12 +330,11 @@ typedef struct r_bin_plugin_t {
 	char *license;
 	int (*init)(void *user);
 	int (*fini)(void *user);
-	Sdb * (*get_sdb)(RBinObject *obj);
-	int (*load)(RBinFile *arch);
+	Sdb * (*get_sdb)(RBinFile *obj);
+	bool (*load)(RBinFile *arch);
 	void *(*load_bytes)(RBinFile *arch, const ut8 *buf, ut64 sz, ut64 loadaddr, Sdb *sdb);
 	ut64 (*size)(RBinFile *bin); // return ut64 maybe? meh
 	int (*destroy)(RBinFile *arch);
-	bool (*check)(RBinFile *arch);
 	bool (*check_bytes)(const ut8 *buf, ut64 length);
 	ut64 (*baddr)(RBinFile *arch);
 	ut64 (*boffset)(RBinFile *arch);
@@ -330,7 +353,7 @@ typedef struct r_bin_plugin_t {
 	RList/*<RBinMem>*/* (*mem)(RBinFile *arch);
 	RList/*<RBinReloc>*/* (*patch_relocs)(RBin *bin);
 	void (*header)(RBinFile *arch);
-	char* (*signature)(RBinFile *arch);
+	char* (*signature)(RBinFile *arch, bool json);
 	int (*demangle_type)(const char *str);
 	struct r_bin_dbginfo_t *dbginfo;
 	struct r_bin_write_t *write;
@@ -358,7 +381,7 @@ typedef struct r_bin_section_t {
 	int bits;
 	bool has_strings;
 	bool add; // indicates when you want to add the section to io `S` command
-	bool is_data;	
+	bool is_data;
 } RBinSection;
 
 typedef struct r_bin_class_t {
@@ -382,6 +405,7 @@ typedef struct r_bin_class_t {
 typedef struct r_bin_symbol_t {
 	/* heap-allocated */
 	char *name;
+	char *dname;
 	char *classname;
 	/* const-unique-strings */
 	const char *forwarder;
@@ -397,6 +421,9 @@ typedef struct r_bin_symbol_t {
 	ut32 ordinal;
 	ut32 visibility;
 	int bits;
+	/* see R_BIN_METH_* constants */
+	ut64 method_flags;
+	int dup_count;
 } RBinSymbol;
 
 typedef struct r_bin_import_t {
@@ -445,6 +472,7 @@ typedef struct r_bin_field_t {
 	char *name;
 	char *comment;
 	char *format;
+	ut64 flags;
 } RBinField;
 
 R_API RBinField *r_bin_field_new(ut64 paddr, ut64 vaddr, int size, const char *name, const char *comment, const char *format);
@@ -485,8 +513,6 @@ typedef struct r_bin_bind_t {
 
 #ifdef R_API
 
-#define r_bin_class_free(x) { free(x->name);free(x->super);free (x); }
-
 /* bin.c */
 R_API void r_bin_load_filter(RBin *bin, ut64 rules);
 R_API int r_bin_load(RBin *bin, const char *file, ut64 baseaddr, ut64 loadaddr, int xtr_idx, int fd, int rawstr);
@@ -540,6 +566,7 @@ R_API bool r_bin_lang_cxx(RBinFile *binfile);
 R_API bool r_bin_lang_msvc(RBinFile *binfile);
 R_API bool r_bin_lang_dlang(RBinFile *binfile);
 R_API bool r_bin_lang_rust(RBinFile *binfile);
+R_API const char *r_bin_get_meth_flag_string(ut64 flag, bool compact);
 
 R_API RList* r_bin_get_entries(RBin *bin);
 R_API RList* r_bin_get_fields(RBin *bin);
@@ -554,6 +581,7 @@ R_API RList* /*<RBinClass>*/r_bin_get_classes(RBin *bin);
 
 R_API RBinClass *r_bin_class_get (RBinFile *binfile, const char *name);
 R_API RBinClass *r_bin_class_new (RBinFile *binfile, const char *name, const char *super, int view);
+R_API void r_bin_class_free(RBinClass *c);
 R_API RBinSymbol *r_bin_class_add_method (RBinFile *binfile, const char *classname, const char *name, int nargs);
 R_API void r_bin_class_add_field (RBinFile *binfile, const char *classname, const char *name);
 
@@ -581,6 +609,7 @@ R_API int r_bin_select(RBin *bin, const char *arch, int bits, const char *name);
 R_API int r_bin_select_idx(RBin *bin, const char *name, int idx);
 R_API int r_bin_select_by_ids(RBin *bin, ut32 binfile_id, ut32 binobj_id );
 R_API int r_bin_object_delete (RBin *bin, ut32 binfile_id, ut32 binobj_id);
+R_API int r_bin_object_set_items(RBinFile *binfile, RBinObject *o);
 R_API int r_bin_use_arch(RBin *bin, const char *arch, int bits, const char *name);
 R_API RBinFile * r_bin_file_find_by_arch_bits(RBin *bin, const char *arch, int bits, const char *name);
 R_API RBinObject * r_bin_object_find_by_arch_bits (RBinFile *binfile, const char *arch, int bits, const char *name);
@@ -650,7 +679,6 @@ extern RBinPlugin r_bin_plugin_mach064;
 extern RBinPlugin r_bin_plugin_java;
 extern RBinPlugin r_bin_plugin_dex;
 extern RBinPlugin r_bin_plugin_dummy;
-extern RBinPlugin r_bin_plugin_rar;
 extern RBinPlugin r_bin_plugin_coff;
 extern RBinPlugin r_bin_plugin_ningb;
 extern RBinPlugin r_bin_plugin_ningba;
@@ -676,6 +704,8 @@ extern RBinPlugin r_bin_plugin_dyldcache;
 extern RBinPlugin r_bin_plugin_avr;
 extern RBinPlugin r_bin_plugin_menuet;
 extern RBinPlugin r_bin_plugin_wasm;
+extern RBinPlugin r_bin_plugin_nro;
+extern RBinPlugin r_bin_plugin_sfc;
 
 #ifdef __cplusplus
 }
