@@ -17,6 +17,7 @@ R_API const char *r_str_rchr(const char *base, const char *p, int ch);
 R_API const char *r_str_closer_chr(const char *b, const char *s);
 R_API int r_str_bounds(const char *str, int *h);
 R_API char *r_str_crop(const char *str, unsigned int x, unsigned int y, unsigned int x2, unsigned int y2);
+R_API bool r_str_range_in(const char *r, ut64 addr);
 R_API int r_str_len_utf8(const char *s);
 R_API int r_str_len_utf8char(const char *s, int left);
 R_API void r_str_filter_zeroline(char *str, int len);
@@ -24,6 +25,7 @@ R_API int r_str_write(int fd, const char *b);
 R_API void r_str_ncpy(char *dst, const char *src, int n);
 R_API void r_str_sanitize(char *c);
 R_API const char *r_str_casestr(const char *a, const char *b);
+R_API const char *r_str_firstbut(const char *s, char ch, const char *but);
 R_API const char *r_str_lastbut(const char *s, char ch, const char *but);
 R_API int r_str_split(char *str, char ch);
 R_API int *r_str_split_lines(char *str, int *count);
@@ -44,7 +46,7 @@ R_API void r_str_argv_free(char **argv);
 R_API char *r_str_new(const char *str);
 R_API int r_str_is_printable(const char *str);
 R_API bool r_str_is_printable_incl_newlines(const char *str);
-R_API char *r_str_concatlen(char *ptr, const char *string, int slen);
+R_API char *r_str_appendlen(char *ptr, const char *string, int slen);
 R_API char *r_str_newf(const char *fmt, ...);
 R_API char *r_str_newlen(const char *str, int len);
 R_API const char *r_str_bool(int b);
@@ -99,6 +101,9 @@ R_API int r_str_re_replace(const char *str, const char *reg, const char *sub);
 R_API int r_str_unescape(char *buf);
 R_API char *r_str_escape(const char *buf);
 R_API char *r_str_escape_dot(const char *buf);
+R_API char *r_str_escape_asciidot(const char *buf);
+R_API char *r_str_escape_latin1(const char *buf);
+R_API char *r_str_escape_utf8(const char *buf);
 R_API void r_str_uri_decode(char *buf);
 R_API char *r_str_uri_encode(const char *buf);
 R_API char *r_str_utf16_decode(const ut8 *s, int len);
@@ -110,9 +115,9 @@ R_API int r_str_nlen_w(const char *s, int n);
 R_API int r_wstr_clen(const char *s);
 R_API char *r_str_prefix(char *ptr, const char *string);
 R_API char *r_str_prefix_all(char *s, const char *pfx);
-R_API char *r_str_concat(char *ptr, const char *string);
-R_API char *r_str_concatf(char *ptr, const char *fmt, ...);
-R_API char *r_str_concatch(char *x, char y);
+R_API char *r_str_append(char *ptr, const char *string);
+R_API char *r_str_appendf(char *ptr, const char *fmt, ...);
+R_API char *r_str_appendch(char *x, char y);
 R_API void r_str_case(char *str, bool up);
 R_API void r_str_chop_path(char *s);
 R_API ut8 r_str_contains_macro(const char *input_value);
@@ -127,5 +132,6 @@ R_API bool r_str_startswith(const char *str, const char *needle);
 R_API bool r_str_endswith(const char *str, const char *needle);
 R_API bool r_str_isnumber (const char *str);
 R_API const char *r_str_last (const char *in, const char *ch);
+R_API char* r_str_highlight(char *str, const char *word, const char *color);
 
 #endif //  R_STR_H
